<?php

namespace Drupal\mason\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\blazy\Plugin\Field\FieldFormatter\BlazyFileFormatterBase;
use Drupal\mason\MasonDefault;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Base class for mason image and file ER formatters.
 */
abstract class MasonFileFormatterBase extends BlazyFileFormatterBase {

  use MasonFormatterTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    return self::injectServices($instance, $container, 'image');
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return MasonDefault::imageSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $entities = $this->getEntitiesToView($items, $langcode);

    // Early opt-out if the field is empty.
    if (empty($entities)) {
      return [];
    }

    return $this->commonViewElements($items, $langcode, $entities);
  }

  /**
   * Build the splide carousel elements.
   */
  public function buildElements(array &$build, $files, $langcode) {
    $settings   = $build['settings'];
    $blazies    = $settings['blazies'];
    $item_id    = $blazies->get('item.id');
    $caption_id = 'caption';
    $elements   = $this->getElements($build, $files, $caption_id);

    foreach ($elements as $element) {
      $captions = $element[$caption_id] ?? [];

      // Do not pass captions to theme_blazy().
      unset($element[$caption_id]);

      // Image with responsive image, lazyLoad, and lightbox supports.
      $element[$item_id] = $this->formatter->getBlazy($element);

      // Build captions if so configured.
      $element[$caption_id] = $captions;

      // Build individual splide item.
      $build['items'][] = $element;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    if (isset($element['image_style'])) {
      $element['image_style']['#description'] = $this->t('This will be treated as the fallback image style if the optionset image styles not provided.');
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function getScopedFormElements() {
    return [
      'background'  => TRUE,
      'breakpoints' => FALSE,
      'no_ratio'    => TRUE,
    ] + $this->getCommonScopedFormElements() + parent::getScopedFormElements();
  }

}
